;Copyright (C) 2006 John T. Haller

;Website: http://PortableApps.com/PortableAppsBackup

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define APPNAME "PortableApps Backup"
!define VER "1.0.0.5"
!define WEBSITE "PortableApps.com/PortableAppsBackup"

;=== Program Details
Name "PortableApps Backup"
OutFile "PortableAppsBackup.exe"
Caption "PortableApps Backup"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${APPNAME}"
VIAddVersionKey Comments "A simple portable backup utility  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com (John T. Haller)"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${APPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${APPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps Backup and PortableApps are trademarks of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "PortableAppsBackup.exe"

;=== Runtime Switches
RequestExecutionLevel user

;=== Includes
!include "MUI.nsh"
!include "Registry.nsh"
!include "GetParent.nsh"
!include "FileFunc.nsh"
!insertmacro GetTime

;=== Icon & Stye ===
!define MUI_ICON "PortableAppsBackup.ico"
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP header.bmp

BrandingText "PortableApps.com - Your Digital Life, Anywhere"
InstallButtonText "Backup >" 
ShowInstDetails show
SubCaption 3 " | Backing Up Files"

;=== Variables
Var DOCUMENTSDIRECTORY
Var BACKUPFILENAME
Var BACKUPDIRECTORY
Var USE7ZFORMAT
Var USEMAXCOMPRESSION
Var SHOWBACKUPDETAILS
Var DRIVELETTER
Var EXECSTRING
Var BACKUPDRIVE
Var BACKUPDOCUMENTS
Var BACKUPDATA
Var FINISHTEXT
Var FINISHTITLE
Var BACKUPEXTENSION
Var BACKUPTYPE
Var PORTABLEAPPSROOTPATH
Var DESKTOPDIRECTORY

;=== Pages
!define MUI_WELCOMEFINISHPAGE_BITMAP welcomefinish.bmp
!define MUI_WELCOMEPAGE_TITLE "Portable Apps Backup"
!define MUI_WELCOMEPAGE_TEXT "Welcome to the Portable Apps Backup utility.\r\n\r\nThis utility allows you to quickly and easily backup the files on your portable device. Just click Next to select the type of backup you wish to perform...\r\n\r\nBackup Tip: As with any backup, it's best to exit all apps while performing a backup."
!insertmacro MUI_PAGE_WELCOME
Page custom ShowBackupWindow LeaveBackupWindow " | Backup Settings" 
Page instfiles
!define MUI_FINISHPAGE_TITLE "$FINISHTITLE"
!define MUI_FINISHPAGE_TEXT "$FINISHTEXT"
!insertmacro MUI_PAGE_FINISH

;=== Languages
!insertmacro MUI_LANGUAGE "English"

Function .onInit
	;=== Variables
	StrCpy $DRIVELETTER $EXEDIR 1
	Push $EXEDIR
	Call GetParent
	Call GetParent
	Pop $PORTABLEAPPSROOTPATH
	${registry::Read} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders" "Personal" $R0 $R1
	StrCpy $DOCUMENTSDIRECTORY $R0
	${registry::Read} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders" "Desktop" $R0 $R1
	StrCpy $DESKTOPDIRECTORY $R0

	!insertmacro MUI_INSTALLOPTIONS_EXTRACT "PortableAppsBackupForm.ini"
	
	;=== Check for settings.ini
	IfFileExists $EXEDIR\Data\settings.ini "BackupType"
		CopyFiles $EXEDIR\App\DefaultData\settings.ini $EXEDIR\Data
	
	BackupType:
		ReadINIStr $BACKUPDRIVE "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "BackupDrive"
		ReadINIStr $BACKUPDATA "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "BackupData"
		ReadINIStr $BACKUPDOCUMENTS "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "BackupDocuments"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 2" "State" "$BACKUPDRIVE"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 3" "State" "$BACKUPDATA"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 4" "State" "$BACKUPDOCUMENTS"

	;=== Calculate Backup Filename
	${GetTime} "" "L" $0 $1 $2 $3 $4 $5 $6
	StrCpy $BACKUPFILENAME "PortableAppsBackup-$2-$1-$0"
	WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 6" "Text" "Backup Directory (Filename: $BACKUPFILENAME-Type)"
    
	;=== Pre-Fill Path with Directory
	ReadINIStr $BACKUPDIRECTORY "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "BackupDirectory"
	StrCmp $BACKUPDIRECTORY "DOCUMENTS\PortableAppsBackup" UseDocumentsPABDirectory CheckDocumentsDirectory
	
	CheckDocumentsDirectory:
		StrCmp $BACKUPDIRECTORY "DOCUMENTS" UseDocumentsDirectory CheckDesktopDirectory

	UseDocumentsPABDirectory:
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 7" "State" "$DOCUMENTSDIRECTORY\PortableAppsBackup"
		Goto BackupOptions

	UseDocumentsDirectory:
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 7" "State" "$DOCUMENTSDIRECTORY"
		Goto BackupOptions
	
	CheckDesktopDirectory:
		StrCmp $BACKUPDIRECTORY "DESKTOP" UseDesktopDirectory UseSavedBackupDirectory

	UseDesktopDirectory:
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 7" "State" "$DESKTOPDIRECTORY"
		Goto BackupOptions

	UseSavedBackupDirectory:
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 7" "State" "$BACKUPDIRECTORY"
	
	BackupOptions:
		ReadINIStr $USE7ZFORMAT "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "Use7zFormat"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 10" "State" "$USE7ZFORMAT"
		ReadINIStr $USEMAXCOMPRESSION "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "UseMaxCompression"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 8" "State" "$USEMAXCOMPRESSION"
		ReadINIStr $SHOWBACKUPDETAILS "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "ShowBackupDetails"
		WriteINIStr $PLUGINSDIR\PortableAppsBackupForm.ini "Field 9" "State" "$SHOWBACKUPDETAILS"
FunctionEnd

Function ShowBackupWindow
!insertmacro MUI_HEADER_TEXT "Portable Apps Backup" "Your Simple Backup Solution for Portable Apps"
    InstallOptions::InitDialog /NOUNLOAD "$PLUGINSDIR\PortableAppsBackupForm.ini"
    Pop $0
    InstallOptions::Show
FunctionEnd

Function LeaveBackupWindow
	;=== Blank
	ReadINIStr $BACKUPDIRECTORY "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 7" "State"
	ReadINIStr $USEMAXCOMPRESSION "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 8" "State"
	ReadINIStr $USE7ZFORMAT "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 10" "State"
	ReadINIStr $SHOWBACKUPDETAILS "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 9" "State"
	ReadINIStr $BACKUPDRIVE "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 2" "State"
	ReadINIStr $BACKUPDATA "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 3" "State"
	ReadINIStr $BACKUPDOCUMENTS "$PLUGINSDIR\PortableAppsBackupForm.ini" "Field 4" "State"
	
	;=== Determine Backup Directory to save
	StrCmp $BACKUPDIRECTORY "$DOCUMENTSDIRECTORY\PortableAppsBackup" "" CheckForDocumentsFromUser
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDirectory" "DOCUMENTS\PortableAppsBackup"
		Goto WriteRemainingSettings	
	
	CheckForDocumentsFromUser:
		StrCmp $BACKUPDIRECTORY $DOCUMENTSDIRECTORY "" CheckForDesktopFromUser
			WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDirectory" "DOCUMENTS"
			Goto WriteRemainingSettings

	CheckForDesktopFromUser:
		StrCmp $BACKUPDIRECTORY $DESKTOPDIRECTORY "" UseFullBackupDirectoryPath
			WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDirectory" "DESKTOP"
			Goto WriteRemainingSettings

	UseFullBackupDirectoryPath:
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDirectory" "$BACKUPDIRECTORY"
	
	WriteRemainingSettings:
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "Use7zFormat" "$USE7ZFORMAT"
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "UseMaxCompression" "$USEMAXCOMPRESSION"
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "ShowBackupDetails" "$SHOWBACKUPDETAILS"
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDrive" "$BACKUPDRIVE"
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupData" "$BACKUPDATA"
		WriteINIStr $EXEDIR\Data\settings.ini "PortableAppsBackup" "BackupDocuments" "$BACKUPDOCUMENTS"
FunctionEnd

Section Main

	;'CompileEXECString:
	StrCpy $EXECSTRING `a -w"$TEMP"`
	
	StrCmp $USE7ZFORMAT "1" "" "UseZipFormat"
		StrCpy $EXECSTRING `$EXECSTRING -t7z`
		StrCpy $BACKUPEXTENSION "7z"
		Goto CheckCompressionType

	UseZipFormat:
		StrCpy $EXECSTRING `$EXECSTRING -tzip`
		StrCpy $BACKUPEXTENSION "zip"
	
	CheckCompressionType:
		StrCmp $USEMAXCOMPRESSION "1" "" CheckBackupDrive
			StrCpy $EXECSTRING `$EXECSTRING -mx7`
	
	CheckBackupDrive:
		StrCmp $BACKUPDRIVE "1" "" CheckBackupData
			StrCpy $BACKUPFILENAME "$BACKUPFILENAME-Drive.$BACKUPEXTENSION"
			StrCpy $EXECSTRING `$EXECSTRING "$BACKUPDIRECTORY\$BACKUPFILENAME" "$DRIVELETTER:\*"`
			StrCpy $BACKUPTYPE "Entire drive ($DRIVELETTER:)"
			SetDetailsPrint none
			CopyFiles /SILENT "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\App\filezilla\FileZilla.xml" "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\Data\settings\FileZilla.xml.backup"
			SetDetailsPrint ListOnly
			Goto CheckForBackupFile
			
	CheckBackupData:
		StrCmp $BACKUPDATA "1" "" CheckBackupDocuments
			StrCpy $BACKUPFILENAME "$BACKUPFILENAME-Data.$BACKUPEXTENSION"
			StrCpy $EXECSTRING `$EXECSTRING "$BACKUPDIRECTORY\$BACKUPFILENAME" "$PORTABLEAPPSROOTPATH\PortableApps\*\Data\*"`
			StrCpy $BACKUPTYPE "Portable app data files"
			SetDetailsPrint none
			CopyFiles /SILENT "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\App\filezilla\FileZilla.xml" "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\Data\settings\FileZilla.xml.backup"
			SetDetailsPrint ListOnly
			Goto CheckForBackupFile

	CheckBackupDocuments:
			StrCpy $BACKUPFILENAME "$BACKUPFILENAME-Documents.$BACKUPEXTENSION"
			StrCpy $EXECSTRING `$EXECSTRING "$BACKUPDIRECTORY\$BACKUPFILENAME" "$PORTABLEAPPSROOTPATH\Documents\*"`
			StrCpy $BACKUPTYPE "Documents ($PORTABLEAPPSROOTPATH\Documents)"
	
	CheckForBackupFile:
		IfFileExists "$BACKUPDIRECTORY\$BACKUPFILENAME" "" SetupProgressBar
		SetDetailsPrint none
			RealProgress::SetProgress /NOUNLOAD 1
			MessageBox MB_YESNO|MB_ICONQUESTION `The file $BACKUPDIRECTORY\$BACKUPFILENAME already exists.  Would you like to replace it?` IDYES DeleteBackupFile
			Goto BackupCancelled
	DeleteBackupFile:
		Delete $BACKUPDIRECTORY\$BACKUPFILENAME
		SetDetailsPrint ListOnly
	
	SetupProgressBar:
		;CheckForDirectory
		IfFileExists "$BACKUPDIRECTORY\*.*" DirectoryExists
			SetDetailsPrint none
			CreateDirectory $BACKUPDIRECTORY
			SetDetailsPrint ListOnly

		DirectoryExists:
			DetailPrint "Backup Type: $BACKUPTYPE"
			DetailPrint "Backup File: $BACKUPDIRECTORY\$BACKUPFILENAME"
			DetailPrint " "
			RealProgress::SetProgress /NOUNLOAD 1
			RealProgress::GradualProgress /NOUNLOAD 2 1 90 "Backup complete."
			DetailPrint "Backing up files..."
			DetailPrint " "
			;MessageBox MB_OK|MB_ICONEXCLAMATION `$EXECSTRING`
			StrCmp $SHOWBACKUPDETAILS 1 BackupWithWindow BackupWithoutWindow
	
	BackupWithWindow:
		DetailPrint "NOTE: The details window is shown.  Closing it will abort the backup."
		DetailPrint " "
		SetDetailsPrint none
		SetOutPath "$EXEDIR\App"
		ExecWait `"$EXEDIR\App\7za.bat" $EXECSTRING`
		SetDetailsPrint ListOnly
		Goto BackupDone
	
	BackupWithoutWindow:
		nsExec::Exec `"$EXEDIR\App\7za.exe" $EXECSTRING`
		Goto BackupDone

	BackupDone:
		DetailPrint " "
		DetailPrint "Backup complete."
		${registry::Unload}
		IfFileExists $BACKUPDIRECTORY\$BACKUPFILENAME BackupSuccessful BackupFailed
		
	BackupSuccessful:
		StrCpy $FINISHTITLE "Backup Complete"
		StrCpy $FINISHTEXT "Your files have been backed up to:\r\n\r\nDirectory:\r\n$BACKUPDIRECTORY\r\n\r\nFilename:\r\n$BACKUPFILENAME"
		Goto TheEnd
		
	BackupFailed:
		StrCpy $FINISHTITLE "Backup Failed"
		StrCpy $FINISHTEXT "The backup process failed to complete.  Please check your configuration."
		Goto TheEnd
		
	BackupCancelled:
		StrCpy $FINISHTITLE "Backup Cancelled"
		StrCpy $FINISHTEXT "The backup process was cancelled."
		
	TheEnd:
SectionEnd

Function .onGUIEnd
	RealProgress::Unload
FunctionEnd