;Copyright (C) 2006 John T. Haller

;Website: http://PortableApps.com/PortableAppsBackup

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define APPNAME "PortableApps Backup - Restore"
!define VER "1.0.0.5"
!define WEBSITE "PortableApps.com/PortableAppsBackup"

;=== Program Details
Name "PortableApps Backup - Restore"
OutFile "PortableAppsBackupRestore.exe"
Caption "PortableApps Backup - Restore"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${APPNAME}"
VIAddVersionKey Comments "A simple portable backup utility  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com (John T. Haller)"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${APPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${APPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps Backup and PortableApps are trademarks of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "PortableAppsBackupRestore.exe"

;=== Runtime Switches
RequestExecutionLevel user

;=== Includes
!include "MUI.nsh"
!include "Registry.nsh"
!include "GetParent.nsh"
!include "FileFunc.nsh"

;=== Icon & Stye ===
!define MUI_ICON "PortableAppsBackup.ico"
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP header.bmp

BrandingText "PortableApps.com - Your Digital Life, Anywhere"
InstallButtonText "Restore >" 
ShowInstDetails show
SubCaption 3 " | Restoring Files"

;=== Variables
Var DOCUMENTSDIRECTORY
Var BACKUPDIRECTORY
Var SHOWBACKUPDETAILS
Var DRIVELETTER
Var EXECSTRING
Var FINISHTEXT
Var FINISHTITLE
Var BACKUPTYPE
Var PORTABLEAPPSROOTPATH
Var DESKTOPDIRECTORY
Var RESTOREFILENAME


;=== Pages
!define MUI_WELCOMEFINISHPAGE_BITMAP welcomefinish.bmp
!define MUI_WELCOMEPAGE_TITLE "Portable Apps Backup - Restore"
!define MUI_WELCOMEPAGE_TEXT "Welcome to the Portable Apps Backup - Restore utility.\r\n\r\nThis utility allows you to quickly and easily restore files to your portable device from backups made with Portable Apps Backup. Just click Next to select the file you wish to restore...\r\n\r\nRestore Tip: As with any restore, you should exit all apps before performing a restore."
!insertmacro MUI_PAGE_WELCOME
Page custom ShowBackupWindow LeaveBackupWindow " | Restore Settings" 
Page instfiles
!define MUI_FINISHPAGE_TITLE "$FINISHTITLE"
!define MUI_FINISHPAGE_TEXT "$FINISHTEXT"
!insertmacro MUI_PAGE_FINISH

;=== Languages
!insertmacro MUI_LANGUAGE "English"

Function .onInit
	;=== Variables
	StrCpy $DRIVELETTER $EXEDIR 1
	Push $EXEDIR
	Call GetParent
	Call GetParent
	Pop $PORTABLEAPPSROOTPATH
	${registry::Read} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders" "Personal" $R0 $R1
	StrCpy $DOCUMENTSDIRECTORY $R0
	${registry::Read} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders" "Desktop" $R0 $R1
	StrCpy $DESKTOPDIRECTORY $R0

	!insertmacro MUI_INSTALLOPTIONS_EXTRACT "PortableAppsBackupRestoreForm.ini"
	
	
	SetOutPath "$DOCUMENTSDIRECTORY"
	
	;=== Check for settings.ini
	IfFileExists $EXEDIR\Data\settings.ini DetermineLastBackupDirectory
		CopyFiles $EXEDIR\App\DefaultData\settings.ini $EXEDIR\Data
	
	;=== Determine last used directory
	DetermineLastBackupDirectory:
		ReadINIStr $BACKUPDIRECTORY "$EXEDIR\Data\settings.ini" "PortableAppsBackup" "BackupDirectory"
		StrCmp $BACKUPDIRECTORY "DOCUMENTS\PortableAppsBackup" UseDocumentsPABDirectory CheckDocumentsDirectory
	
	CheckDocumentsDirectory:
		StrCmp $BACKUPDIRECTORY "DOCUMENTS" UseDocumentsDirectory CheckDesktopDirectory

	UseDocumentsPABDirectory:
		StrCpy $BACKUPDIRECTORY "$DOCUMENTSDIRECTORY\PortableAppsBackup"
		Goto onInitEnd

	UseDocumentsDirectory:
		StrCpy $BACKUPDIRECTORY "$DOCUMENTSDIRECTORY"
		Goto onInitEnd
	
	CheckDesktopDirectory:
		StrCmp $BACKUPDIRECTORY "DESKTOP" UseDesktopDirectory UseSavedBackupDirectory

	UseDesktopDirectory:
		StrCpy $BACKUPDIRECTORY "$DESKTOPDIRECTORY"
		Goto onInitEnd

	UseSavedBackupDirectory:
		StrCpy $BACKUPDIRECTORY "$BACKUPDIRECTORY" ;== kept in case we end up adding a transform or sep variable later
	
	onInitEnd:
		SetOutPath "$BACKUPDIRECTORY"
FunctionEnd

Function ShowBackupWindow
	!insertmacro MUI_HEADER_TEXT "Portable Apps Backup" "Your Simple Backup Solution for Portable Apps"
    InstallOptions::InitDialog /NOUNLOAD "$PLUGINSDIR\PortableAppsBackupRestoreForm.ini"
    Pop $0
    InstallOptions::Show
FunctionEnd

Function LeaveBackupWindow
	ReadINIStr $RESTOREFILENAME "$PLUGINSDIR\PortableAppsBackupRestoreForm.ini" "Field 3" "State"
	ReadINIStr $SHOWBACKUPDETAILS "$PLUGINSDIR\PortableAppsBackupRestoreForm.ini" "Field 5" "State"
	
	;=== Determine backup type from filename
	StrCpy $0 $RESTOREFILENAME "" -7
	StrCmp $0 "Data.7z" TypeDataBackup
	StrCmp $0 "ata.zip" TypeDataBackup
	StrCmp $0 "rive.7z" TypeDriveBackup
	StrCmp $0 "ive.zip" TypeDriveBackup
	StrCmp $0 "ents.7z" TypeDocumentsBackup
	StrCmp $0 "nts.zip" TypeDocumentsBackup
	MessageBox MB_OK|MB_ICONEXCLAMATION `Could not determine the type of backup from the file name.  You will have to manually restore this file.$\n$\nSince it is a standard archive file, you can use an application like 7-Zip Portable to open it.`
	Abort
	
	TypeDataBackup:
		StrCpy $BACKUPTYPE "Data"
		Goto LeaveBackupWindowEnd

	TypeDriveBackup:
		StrCpy $BACKUPTYPE "Drive"
		Goto LeaveBackupWindowEnd
		
	TypeDocumentsBackup:
		StrCpy $BACKUPTYPE "Documents"
		Goto LeaveBackupWindowEnd

	LeaveBackupWindowEnd:
FunctionEnd

Section Main
	;'CompileEXECString:
	StrCpy $EXECSTRING `x "$RESTOREFILENAME"`
	
	StrCmp $BACKUPTYPE "Data" RestoreData
	StrCmp $BACKUPTYPE "Documents" RestoreDocuments
	StrCmp $BACKUPTYPE "Drive" RestoreDrive

	RestoreData:
		StrCpy $EXECSTRING `$EXECSTRING -x!PortableAppsBackup* -o$PORTABLEAPPSROOTPATH\PortableApps`
		Goto SetupProgressBar

	RestoreDocuments:
		StrCpy $EXECSTRING `$EXECSTRING -o$PORTABLEAPPSROOTPATH\Documents`
		Goto SetupProgressBar

	RestoreDrive:
		StrCpy $EXECSTRING `$EXECSTRING -x!PortableApps\PortableAppsBackup* -o$DRIVELETTER:\`
		Goto SetupProgressBar

	SetupProgressBar:
		StrCpy $EXECSTRING `$EXECSTRING -aoa`

		DetailPrint "Restore Type: $BACKUPTYPE"
		DetailPrint "Restore File: $RESTOREFILENAME"
		DetailPrint " "
		RealProgress::SetProgress /NOUNLOAD 1
		RealProgress::GradualProgress /NOUNLOAD 2 1 90 "Restore complete."
		DetailPrint "Restoring files..."
		DetailPrint " "
		;MessageBox MB_OK|MB_ICONEXCLAMATION `$EXECSTRING`
		StrCmp $SHOWBACKUPDETAILS 1 BackupWithWindow BackupWithoutWindow
	
	BackupWithWindow:
		DetailPrint "NOTE: The details window is shown.  Closing it will abort the restore."
		DetailPrint " "
		SetDetailsPrint none
		SetOutPath "$EXEDIR\App"
		ExecWait `"$EXEDIR\App\7za.bat" $EXECSTRING`
		SetDetailsPrint ListOnly
		Goto BackupDone
	
	BackupWithoutWindow:
		nsExec::Exec `"$EXEDIR\App\7za.exe" $EXECSTRING`
		Goto BackupDone

	BackupDone:
		DetailPrint " "
		DetailPrint "Restore complete."
		SetDetailsPrint none
		${registry::Unload}
		;IfFileExists $BACKUPDIRECTORY\$BACKUPFILENAME BackupSuccessful BackupFailed
	
		StrCmp $BACKUPTYPE "Data" FileZillaFixPost
		StrCmp $BACKUPTYPE "Documents" DocumentsDirFixPost
		StrCmp $BACKUPTYPE "Drive" FileZillaFixPost
		
		FileZillaFixPost:
			SetDetailsPrint none
			CopyFiles /SILENT "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\Data\settings\FileZilla.xml.backup" "$PORTABLEAPPSROOTPATH\PortableApps\FileZillaPortable\App\filezilla\FileZilla.xml"
			SetDetailsPrint ListOnly
			Goto BackupSuccessful
			
		DocumentsDirFixPost:
			SetOutPath "$DRIVELETTER:\"
			nsExec::Exec `attrib +R $DRIVELETTER:\Documents`
			Goto BackupSuccessful
	
	BackupSuccessful:
		StrCpy $FINISHTITLE "Restore Complete"
		StrCpy $FINISHTEXT "The files have been restored.  If you plan on deleting the backup file, please check that all the files you expected to be restored have been before doing so."
		Goto TheEnd
		
	;BackupFailed:
		;StrCpy $FINISHTITLE "Backup Failed"
		;StrCpy $FINISHTEXT "The backup process failed to complete.  Please check your configuration."
		;Goto TheEnd
		
	;BackupCancelled:
		;StrCpy $FINISHTITLE "Restore Cancelled"
		;StrCpy $FINISHTEXT "The restore process was cancelled."
		
	TheEnd:
SectionEnd

Function .onGUIEnd
	RealProgress::Unload
FunctionEnd